use crate::download_wrapper::DownloadType;
use anyhow::Error;
use mysql::prelude::*;

pub(crate) struct MesService {
    pool: mysql::Pool,
}

impl MesService {
    pub fn new(
        ip: String,
        port: String,
        username: String,
        password: String,
        database: String,
    ) -> Result<Self, Error> {
        let url = format!(
            "mysql://{}:{}@{}:{}/{}",
            username, password, ip, port, database
        );
        let pool = mysql::Pool::new(url.as_str())?;
        Ok(Self { pool })
    }

    pub fn get_work_orders(&self) -> Result<Vec<String>, Error> {
        let mut conn = self.pool.get_conn()?;
        let work_orders: Vec<String> =
            conn.query("SELECT `OrderId` FROM wisun_ordertables".to_string())?;
        Ok(work_orders)
    }

    pub fn check_station(
        &self,
        work_order: String,
        label: String,
        download_type: DownloadType,
    ) -> Result<bool, Error> {
        match download_type {
            DownloadType::Rail => {
                let mut conn = self.pool.get_conn()?;
                let check_result: Vec<String> = conn.query(format!(
                    "SELECT `Station2` FROM `{work_order}` WHERE `Barcode` = '{label}'"
                ))?;
                if check_result.is_empty() {
                    return Ok(false);
                }
                Ok(check_result[0] == "1")
            }
            DownloadType::App => {
                let mut conn = self.pool.get_conn()?;
                let check_result: Vec<String> = conn.query(format!(
                    "SELECT `IMEI_1` FROM `{work_order}` WHERE `Barcode` = '{label}';"
                ))?;
                if check_result.is_empty() {
                    Ok(true)
                } else {
                    Ok(false)
                }
            }
            DownloadType::Bootloader => Ok(true),
        }
    }

    pub fn get_order_number(&self, work_order: String)->Result<String, Error>{
        let mut conn = self.pool.get_conn()?;
        let order_number: Vec<String> = conn.query(format!(
            "SELECT `Batch` FROM wisun_ordertables WHERE `OrderId` = '{work_order}'"
        ))?;
        if order_number.is_empty() {
            Err(Error::msg("No order number found"))
        }else{
            Ok(order_number[0].clone())
        }
    }

    pub fn update_station(
        &self,
        work_order: String,
        label: String,
        download_type: DownloadType,
    ) -> Result<(), Error> {
        let time_now = chrono::Local::now().format("%Y-%m-%d %H:%M:%S").to_string();
        match download_type {
            DownloadType::Bootloader => {}
            DownloadType::App => {
                let mut conn = self.pool.get_conn()?;
                let work_order_id: Vec<u32> = conn.query(format!(
                    "SELECT `id` FROM wisun_ordertables WHERE `OrderId` = '{work_order}'"
                ))?;
                if let Some(work_order_id) = work_order_id.first() {
                    let _: Vec<String> = conn.exec(
                        format!(
                            "INSERT INTO `{work_order}` (`ID`, `Barcode`, `IMEI_1`, `Station1`, `Station1Time`) VALUES ( {work_order_id}, '{label}', '{label}', '1', '{time_now}') \
        ON DUPLICATE KEY UPDATE `Barcode` = VALUES(`Barcode`), `IMEI_1` = VALUES(`IMEI_1`), `Station1` = VALUES(`Station1`), `Station1Time`= VALUES(`Station1Time`)",
                            work_order_id = work_order_id,
                            label = label,
                            time_now = time_now
                        ),
                        ()
                    )?;
                } else {
                    return Ok(());
                }
            }
            DownloadType::Rail => {
                let mut conn = self.pool.get_conn()?;
                let _: Vec<String> = conn.exec(
                    format!("UPDATE `{work_order}` SET Station3 = 1, Station3Time='{time_now}' WHERE Barcode = '{label}'"),
                    (),
                )?;
            }
        }
        Ok(())
    }

    #[cfg(test)]
    fn reset(&self) {
        let mut conn = self.pool.get_conn().unwrap();
        let _: Vec<String> = conn
            .query(
                "UPDATE rrr SET `Station1` = 0, `Station2` = 0, `Station3` = 0 WHERE `Barcode` = 1"
                    .to_string(),
            )
            .unwrap();
    }
}

#[cfg(test)]
mod test {
    use crate::mes_service::MesService;
    use mysql::prelude::*;

    #[test]
    fn test_mes_service() {
        let mes = MesService::new(
            "192.168.1.17".to_string(),
            "3306".to_string(),
            "root".to_string(),
            "".to_string(),
            "MOBILETEK_WISUN".to_string(),
        );
    }

    #[test]
    fn test_get_work_orders() {
        let mes = MesService::new(
            "192.168.1.17".to_string(),
            "3306".to_string(),
            "root".to_string(),
            "".to_string(),
            "MOBILETEK_WISUN".to_string(),
        );
        let work_orders = mes.unwrap().get_work_orders().unwrap();
        assert_ne!(work_orders.len(), 0);
        println!("{:?}", work_orders);
    }

    #[test]
    fn test_check_station() {
        let mes = MesService::new(
            "192.168.1.17".to_string(),
            "3306".to_string(),
            "root".to_string(),
            "".to_string(),
            "MOBILETEK_WISUN".to_string(),
        );
        let mut conn = mes.unwrap().pool.get_conn().unwrap();
        let work_order = "3333";
        let label = "202400154Y000016";
        let check_result: Vec<String> = conn
            .query(format!(
                "SELECT `COUNT` FROM `{work_order}` WHERE `Barcode` = '{label}';"
            ))
            .unwrap();
        println!("{:?}", check_result);
    }
}
